/*
    eondump.c

    Converts EON raw files produced by eonlink into UDCF format.

    Copyright (C) 2001 Matthias Cramer, cramer@freestone.net
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include "version.h"
#include "config.h"

extern int errno;

int main(void) {
  int fd;
  FILE * pf;
  char filename[255];
  char dirname[100];
  char eonbuf[2306];
  char * p_eonbuf = eonbuf;
  int i, j, divenum, first;
  int depth;
  
  struct s_eonglobal {
    unsigned int n_dives;
    int divetime;
    float maxdepth;
    int offset;
    int endpoint;
    int interval;
    int altitude;
    int altmode;
    int year;
    char owner[21];
    int serial;
  } eonglobal;
 
  struct s_eondive {
    unsigned int divenum;
    unsigned char surfmin;
    unsigned char surfhour;
    unsigned char repetitive;
    unsigned char mystery;
    unsigned char sampleintervall;
    unsigned char altitude;
    unsigned char altmode;
    int alt;
    unsigned char startpresure;
    int  year;
    unsigned char month;
    unsigned char day;
    unsigned char hour;
    unsigned char min;
    int divetime;
    unsigned char temp;
    unsigned char endpresure;
    char diveprofile[630];
    int profilesamples;
  };

  struct s_eondive dive[32];
  
  struct stat dumpstat;

  struct s_config config;
  readconfig(&config);
  
  printf("eondump V%s %s\n", VERSION, COPY);
  
  stat("eondump.raw", &dumpstat);

  if (dumpstat.st_size != 2305) {
    printf("The file 'eondump.raw' has the wrong size or does not exist!");
    return(1);
  }

  fd = open("eondump.raw", O_RDONLY);
  read(fd, eonbuf, 2305);
  close(fd);
  
  eonglobal.n_dives  = ((int)(eonbuf[0])<<16) + ((int)(eonbuf[1])<<8) + ((int)(eonbuf[2]));
  eonglobal.divetime = ((int)(eonbuf[3])<<8) + ((int)eonbuf[4]);
  eonglobal.maxdepth = (float)(((((int)(eonbuf[5])<<8) + ((int)eonbuf[6])) * 1.0) / 128.0) * 0.3048;
  eonglobal.offset   = 0x100 + (int)((int)(eonbuf[7])<<8) + ((int)eonbuf[8]);
  eonglobal.interval = eonbuf[9];
  eonglobal.altitude = (eonbuf[10] & 0x70) >> 4;
  eonglobal.year     = (((eonbuf[11] & 0xF0)>>4)*10) + (eonbuf[11] & 0x0F);
  eonglobal.year     = (eonglobal.year < 70) ? (eonglobal.year + 2000) : (eonglobal.year + 1900);
  strncpy(eonglobal.owner,p_eonbuf + 12,20);
  eonglobal.owner[20]=0;
  eonglobal.serial   = (((eonbuf[244] & 0xF0)>>4) *100000) + ((eonbuf[244] & 0x0F) *10000) + 
                       (((eonbuf[245] & 0xF0)>>4) * 1000) + ((eonbuf[245] & 0x0F) * 100) + 
                       (((eonbuf[246] & 0xF0)>>4) * 10) + (eonbuf[246] & 0x0F);

  switch (eonglobal.altitude) {
  	case 1: 
  	   eonglobal.altmode=0;
  	   break;
  	case 3: 
  	   eonglobal.altmode=1;
  	   break;
  	case 7: 
  	   eonglobal.altmode=2;
  	   break;
  }

  printf("Dives: %d\n",eonglobal.n_dives);
  printf("Total divetime: %d min\n", eonglobal.divetime);
  printf("Max depth: %5.2f  m\n", eonglobal.maxdepth);
  printf("Current interval: %ds\n", eonglobal.interval);
  printf("Altitude setting: A%d\n", eonglobal.altmode);
  printf("Year: %d\n", eonglobal.year);
  printf("Owner: %s\n", eonglobal.owner);
  printf("Serial: %d\n", eonglobal.serial);
  
  printf("\nOffset: %d (0x%x)\n", eonglobal.offset, eonglobal.offset);

  printf("---------------------------------------------------------\n");
  i=0x100; 
  j=i-3;
   
  while ((eonbuf[i] != (char)0x82) && (eonbuf[j] != (char) 0x80)) {
    i++;
    j++;
    if(i>0x8ff) {i=0x100;}
    if(j>0x8ff) {j=0x100;}
  }

  divenum=0;
  first=1;
  
  while (i!=eonglobal.endpoint) {

    dive[divenum].surfmin=eonbuf[i++];
    if(i>0x8ff) {i=0x100;}
    dive[divenum].surfhour=eonbuf[i++];
    if(i>0x8ff) {i=0x100;}
    dive[divenum].repetitive=eonbuf[i++];
    if(i>0x8ff) {i=0x100;}
    
    dive[divenum].sampleintervall=eonbuf[i++];
    if(i>0x8ff) {i=0x100;}
    dive[divenum].altitude=(eonbuf[i++] & 0x70) >> 4;
    if(i>0x8ff) {i=0x100;}
    dive[divenum].startpresure=eonbuf[i++]*2;
    if(i>0x8ff) {i=0x100;}
    dive[divenum].year=(((eonbuf[i] & 0xF0)>>4)*10) + (eonbuf[i++] & 0x0F);
    if(i>0x8ff) {i=0x100;}
    dive[divenum].year = (dive[divenum].year < 70) ? (dive[divenum].year + 2000) : (dive[divenum].year + 1900);
    if(i>0x8ff) {i=0x100;}
    dive[divenum].month=(((eonbuf[i] & 0xF0)>>4)*10) + (eonbuf[i++] & 0x0F);
    if(i>0x8ff) {i=0x100;}
    dive[divenum].day=(((eonbuf[i] & 0xF0)>>4)*10) + (eonbuf[i++] & 0x0F);
    if(i>0x8ff) {i=0x100;}
    dive[divenum].hour=(((eonbuf[i] & 0xF0)>>4)*10) + (eonbuf[i++] & 0x0F);
    if(i>0x8ff) {i=0x100;}
    dive[divenum].min=(((eonbuf[i] & 0xF0)>>4)*10) + (eonbuf[i++] & 0x0F);
    if(i>0x8ff) {i=0x100;}

    j=0;

    while (eonbuf[i] != (char) 0x80) {
      dive[divenum].diveprofile[j]=eonbuf[i];
      i++;
      if(i>0x8ff) {i=0x100;}
      j++;
    }

    dive[divenum].divetime=(j-1)*dive[divenum].sampleintervall/60;
    dive[divenum].profilesamples=j-1;


    i++; 
    if(i>0x8ff) {i=0x100;}

    dive[divenum].temp= eonbuf[i++] -40;
    if(i>0x8ff) {i=0x100;}
    dive[divenum].endpresure=eonbuf[i++] * 2;
    if(i>0x8ff) {i=0x100;}

    divenum++;
    
    if ((first == 1) && (eonbuf[i] == (char) 0x82)) {
      eonglobal.endpoint=i;
      divenum=0; 
      first=0;
      i++;
     if(i>0x8ff) {i=0x100;}
    }
  }

  for (i=1; i<divenum; i++) {
    
    switch (dive[i].altitude) {
  	case 1: 
  	   dive[i].altmode=0;
  	   dive[i].alt=0;
  	   break;
  	case 3: 
  	   dive[i].altmode=1;
  	   dive[i].alt=700;
  	   break;
  	case 7: 
  	   dive[i].altmode=2;
  	   dive[i].alt=1500;
  	   break;
    }

    dive[i].divenum=eonglobal.n_dives-divenum+i+1+config.diveoffset;
    
    /* Look if ".eon" does exist */

    sprintf(dirname, "%s/.eon",getenv("HOME"));
    
    stat(dirname, &dumpstat);
    
    if ((dumpstat.st_mode & S_IFDIR) != S_IFDIR) {
      printf("Create .eon directory ...");
      mkdir(dirname, 0755);
      printf(" done\n");
    }
        
    sprintf(filename, "%s/%d-%04d-%02d-%02d_%02d%02d.udcf", dirname, dive[i].divenum, dive[i].year, dive[i].month, dive[i].day, dive[i].hour, dive[i].min);
    if (open(filename,O_RDONLY) == -1) {
      pf = fopen(filename,"w");
      if (pf == NULL) {
        fprintf(stderr, "Error: Can not open UDCF file!\n");
        return(1);
      }
      printf("Dive will be saved in UDCF format as %s\n", filename);
      printf("Dive number: %d\n", dive[i].divenum);
      printf("Surface interval: %02d:%02d\n", dive[i].surfhour, dive[i].surfmin);
      printf("Repetition no.: %d\n", dive[i].repetitive);
      printf("Sample interval: %02ds\n", dive[i].sampleintervall);
      printf("Altitude setting: A%d\n",dive[i].altmode);
      printf("Start pressure: %d Bar\n",dive[i].startpresure);
      printf("Dive start: %02d.%02d.%04d %02d:%02d\n", dive[i].day, dive[i].month, dive[i].year, dive[i].hour, dive[i].min);
      printf("Divetime: %d min\n", dive[i].divetime);
      printf("Temperature: %02d C\n", dive[i].temp);
      printf("End pressure: %02d Bar\n", dive[i].endpresure);
      printf("-------------------------------------------------------\n");
      
      /* Write the data out in UDCF format */
      
      fprintf(pf, "<PROFILE UDCF=\"1\">\n");
      fprintf(pf, "<UNITS>Metric</UNITS>\n");
      fprintf(pf, "<DEVICE>\n <VENDOR>Suunto</VENDOR>\n <MODEL>EON</MODEL>\n <VERSION>1</VERSION>\n</DEVICE>\n");
      fprintf(pf, "<ADDINFO>\n");
      fprintf(pf, " <PERSINFO>%s</PERSINFO>\n", eonglobal.owner);
      fprintf(pf, " <TOTALDIVES>%d</TOTALDIVES>\n", dive[i].divenum);
      fprintf(pf, " <SERIALID>%d</SERIALID>\n", eonglobal.serial);
      fprintf(pf, "</ADDINFO>\n");
      fprintf(pf, "<REPGROUP>\n");
      fprintf(pf, " <DIVE>\n");
      fprintf(pf, "  <DATE><DAY>%d</DAY><MONTH>%d</MONTH><YEAR>%04d</YEAR></DATE>\n", dive[i].day, dive[i].month, dive[i].year);
      fprintf(pf, "  <TIME><HOUR>%d</HOUR><MINUTE>%d</MINUTE></TIME>\n", dive[i].hour, dive[i].min);
      fprintf(pf, "  <SURFACEINTERVAL>%d</SURFACEINTERVAL>\n", (dive[i].repetitive <= 1) ? 0 : dive[i].surfhour*60 + dive[i].surfmin);
      fprintf(pf, "  <TEMPERATURE>%d</TEMPERATURE>\n", dive[i].temp);
      fprintf(pf, "  <DENSITY>1030.000</DENSITY>\n");
      fprintf(pf, "  <ALTITUDE>%d</ALTITUDE>\n",dive[i].alt);
      fprintf(pf, "  <GASES>\n");
      fprintf(pf, "   <MIX>\n");
      fprintf(pf, "    <MIXNAME>RealAir</MIXNAME>\n");
      fprintf(pf, "    <TANKVOLUME>15.0</TANKVOLUME>\n");
      fprintf(pf, "    <TANK><PSTART>%d00000.0</PSTART><PEND>%d00000.0</PEND></TANK>\n", dive[i].startpresure, dive[i].endpresure);
      fprintf(pf, "    <O2>0.21</O2>\n");
      fprintf(pf, "    <N2>0.79</N2>\n");
      fprintf(pf, "    <HE>0.00</HE>\n");
      fprintf(pf, "   </MIX>\n");
      fprintf(pf, "  </GASES>\n");
      fprintf(pf, "  <DELTAMODE/>\n");
      fprintf(pf, "  <SAMPLES>\n");
      fprintf(pf, "   <DELTA>%d</DELTA>\n", dive[i].sampleintervall);
      fprintf(pf, "   <SWITCH>RealAir</SWITCH>\n");
      
      depth=0;
      for (j=0; j<=dive[i].profilesamples; j++) {
        switch ((unsigned char)dive[i].diveprofile[j]) {
          case 0x7e:  /* Decompression Dive */
            fprintf(pf, "   <ALARM>DECO</ALARM>\n");
            break;
          case 0x7f:  /* Missed Deco Step */
            fprintf(pf, "   <ALARM>DECO MISSED</ALARM>\n");
            break;
          case 0x81:  /* Slow warning for the entire dive */
            fprintf(pf, "   <ALARM>ASCENT</ALARM>\n");
            break;
            case 0x80:  /* End of dive */
            fprintf(pf, "   <ALARM>END</ALARM>\n");
            break;
          case 0x7d:  /* Surfaced */
            fprintf(pf, "   <ALARM>SURFACE</ALARM>\n");
            break;
          default:
            depth=depth+(char)dive[i].diveprofile[j];
            fprintf(pf, "   <D>%6.2f</D>\n", (float)(depth * 0.3048));
        }
      }
      fprintf(pf, "  </SAMPLES>\n");
      fprintf(pf, " </DIVE>\n");
      fprintf(pf, "</REPGROUP>\n");
      fprintf(pf, "</PROFILE>\n");      
      
      fclose(pf);
    } else {
      printf("Dive '%s' already exists, skipping [%s (%d)] ...\n", filename, strerror(errno), errno);
    }
  }
  return(0);
}
